import "dart:convert";
import 'package:flutter/rendering.dart';

import "../models/dart/entity.pb.dart";
import "../models/dart/feed.pb.dart";

// ////////////////////////////////////////////////////////////////////////////
// ENTITY
// ////////////////////////////////////////////////////////////////////////////

Entity parseEntity(String json) {
  try {
    final mapEntity = jsonDecode(json);
    if (!(mapEntity is Map)) throw "The entity metadata is invalid";

    final Entity entity = Entity();
    entity.version = mapEntity["version"] ?? "";
    if (mapEntity["languages"] != null)
      entity.languages.addAll(mapEntity["languages"].cast<String>() ?? []);
    entity.entityId = mapEntity["entityId"] ?? "";
    if (mapEntity["name"] != null)
      entity.name.addAll(mapEntity["name"].cast<String, String>() ?? {});
    if (mapEntity["description"] != null)
      entity.description
          .addAll(mapEntity["description"].cast<String, String>() ?? {});

    if (mapEntity["contracts"] != null) {
      final Entity_ContractsInfo contracts = Entity_ContractsInfo();
      if (mapEntity["contracts"]["resolverAddress"] != null)
        contracts.resolverAddress =
            mapEntity["contracts"]["resolverAddress"] ?? "";
      if (mapEntity["contracts"]["votingAddress"] != null)
        contracts.votingAddress = mapEntity["contracts"]["votingAddress"] ?? "";
      if (mapEntity["contracts"]["networkId"] != null)
        contracts.networkId = mapEntity["contracts"]["networkId"] ?? "";
      entity.contracts = contracts;
    }

    if (mapEntity["votingProcesses"] != null) {
      Entity_VotingProcesses votingProcesses = Entity_VotingProcesses();
      if (mapEntity["votingProcesses"]["active"] != null)
        votingProcesses.active.addAll(
            mapEntity["votingProcesses"]["active"].cast<String>() ?? []);
      if (mapEntity["votingProcesses"]["ended"] != null)
        votingProcesses.ended
            .addAll(mapEntity["votingProcesses"]["ended"].cast<String>() ?? []);
      entity.votingProcesses = votingProcesses;
    }

    if (mapEntity["newsFeed"] != null)
      entity.newsFeed
          .addAll(mapEntity["newsFeed"].cast<String, String>() ?? {});

    if (mapEntity["media"] != null) {
      Entity_Media media = Entity_Media();
      if (mapEntity["media"]["avatar"] != null)
        media.avatar = mapEntity["media"]["avatar"] ?? "";
      if (mapEntity["media"]["header"] != null)
        media.header = mapEntity["media"]["header"] ?? "";
      entity.media = media;
    }

    final actions = parseEntityActions(mapEntity["actions"]);
    entity.actions.addAll(actions);

    final bootNodes = parseBootNodes(mapEntity["bootNodes"]);
    entity.gatewayBootNodes.addAll(bootNodes);

    if (mapEntity["gatewayUpdate"] != null) {
      Entity_GatewayUpdate gatewayUpdate = Entity_GatewayUpdate();
      if (mapEntity["gatewayUpdate"]["timeout"] != null)
        gatewayUpdate.timeout = mapEntity["gatewayUpdate"]["timeout"] ?? 60000;
      if (mapEntity["gatewayUpdate"]["topic"] != null)
        gatewayUpdate.topic = mapEntity["gatewayUpdate"]["topic"] ?? "";
      if (mapEntity["gatewayUpdate"]["difficulty"] != null)
        gatewayUpdate.difficulty =
            mapEntity["gatewayUpdate"]["difficulty"] ?? 1000;
      entity.gatewayUpdate = gatewayUpdate;
    }

    final relays = parseRelay(mapEntity["relays"]);
    entity.relays.addAll(relays);

    final bootEntities = parseEntityReference(mapEntity["bootEntities"]);
    entity.bootEntities.addAll(bootEntities);

    final fallbackBootNodeEntities =
        parseEntityReference(mapEntity["fallbackBootNodeEntities"]);
    entity.fallbackBootNodeEntities.addAll(fallbackBootNodeEntities);

    final trustedEntities = parseEntityReference(mapEntity["trustedEntities"]);
    entity.trustedEntities.addAll(trustedEntities);

    final censusServiceManagedEntities =
        parseEntityReference(mapEntity["censusServiceManagedEntities"]);
    entity.censusServiceManagedEntities.addAll(censusServiceManagedEntities);

    return entity;
  } catch (err) {
    throw FlutterError("The entity metadata could not be parsed");
  }
}

List<Entity_Action> parseEntityActions(List actions) {
  if (!(actions is List)) return [];
  return actions.whereType<Map>().map((action) {
    Entity_Action result = Entity_Action();
    result.type = action["type"];
    result.name.addAll(action["name"].cast<String, String>() ?? {});
    result.visible = action["visible"];
    // type = browser / image
    result.url = action["url"];
    // type = image
    if (action["imageSources"] is List) {
      final sources = action["imageSources"].whereType<Map>().map((source) {
        Entity_Action_ImageSource result = Entity_Action_ImageSource();
        result.type = source["type"] ?? "";
        result.name = source["name"] ?? "";
        result.orientation = source["orientation"] ?? "";
        result.overlay = source["overlay"] ?? "";
        result.caption.addAll(source["caption"].cast<String, String>() ?? {});
        return result;
      });
      result.imageSources.addAll(sources);
    }
    return result;
  }).toList();
}

List<Entity_GatewayBootNode> parseBootNodes(List bootNodes) {
  if (!(bootNodes is List)) return [];
  return bootNodes.whereType<Map>().map((node) {
    Entity_GatewayBootNode result = Entity_GatewayBootNode();
    result.fetchUri = node["fetchUri"];
    result.heartbeatMessagingUri = node["heartbeatMessagingUri"];
    return result;
  }).toList();
}

List<Entity_Relay> parseRelay(List relays) {
  if (!(relays is List)) return [];
  return relays.whereType<Map>().map((relay) {
    Entity_Relay result = Entity_Relay();
    result.publicKey = relay["publicKey"];
    result.messagingUri = relay["messagingUri"];
    return result;
  }).toList();
}

List<Entity_EntityReference> parseEntityReference(List entities) {
  if (!(entities is List)) return [];
  return entities.whereType<Map>().map((entity) {
    Entity_EntityReference result = Entity_EntityReference();
    result.resolverAddress = entity["resolverAddress"];
    result.networkId = entity["networkId"];
    result.entityId = entity["entityId"];
    return result;
  }).toList();
}

// ////////////////////////////////////////////////////////////////////////////
// FEED
// ////////////////////////////////////////////////////////////////////////////

Feed parseFeed(String json) {
  try {
    Feed result = Feed();
    final mapFeed = jsonDecode(json);
    if (!(mapFeed is Map)) return null;

    if (mapFeed["version"] != null) result.version = mapFeed["version"];
    if (mapFeed["title"] != null) result.title = mapFeed["title"];
    if (mapFeed["home_page_url"] != null)
      result.homePageUrl = mapFeed["home_page_url"];
    if (mapFeed["description"] != null)
      result.description = mapFeed["description"];
    if (mapFeed["feed_url"] != null) result.feedUrl = mapFeed["feed_url"];
    if (mapFeed["icon"] != null) result.icon = mapFeed["icon"];
    if (mapFeed["favicon"] != null) result.favicon = mapFeed["favicon"];
    result.expired = mapFeed["expired"] ?? false;

    if (mapFeed["items"] is List) {
      List<FeedPost> items =
          (mapFeed["items"] as List).whereType<Map>().map((item) {
        FeedPost post = FeedPost();
        if (item["id"] != null) post.id = item["id"];
        if (item["title"] != null) post.title = item["title"];
        if (item["summary"] != null) post.summary = item["summary"];
        if (item["content_text"] != null)
          post.contentText = item["content_text"];
        if (item["content_html"] != null)
          post.contentHtml = item["content_html"];
        if (item["url"] != null) post.url = item["url"];
        if (item["image"] != null) post.image = item["image"];
        if (item["tags"] != null)
          post.tags.addAll((item["tags"] as List).cast<String>());
        if (item["date_published"] != null)
          post.datePublished = item["date_published"];
        if (item["date_modified"] != null)
          post.dateModified = item["date_modified"];

        FeedPost_Author author = FeedPost_Author();
        if (item["author"] != null) {
          if (item["author"]["name"] != null)
            author.name = item["author"]["name"];
          if (item["author"]["url"] != null) author.url = item["author"]["url"];
          post.author = author;
        }
        return post;
      }).toList();
      result.items.addAll(items);
    }

    return result;
  } catch (err) {
    throw FlutterError("The news feed could not be parsed");
  }
}
